@@ -32,6 +32,8 @@
 #include <sys/stat.h>
 #include <pwd.h>
 #include <sys/types.h>
+#include <signal.h>
+
 #include "gdm.h"
 #include "misc.h"
 #include "auth.h"
@@ -49,6 +51,7 @@ extern char *stored_path;
 extern gchar *GdmUser;
 extern gchar *GdmServAuthDir;
 extern gchar *GdmGtkRC;
+extern gchar *GdmGtkTheme;
 extern uid_t GdmUserId;
 extern gid_t GdmGroupId;
 extern gboolean GdmAddGtkModules;
@@ -260,10 +263,28 @@ setup_dialog (GdmDisplay *d, const char *name, int closefdexcept, gboolean set_g
 
 	gtk_init (&argc, &argv);
 
-	if( ! inhibit_gtk_themes &&
-	    ! ve_string_empty (GdmGtkRC) &&
-	   access (GdmGtkRC, R_OK) == 0)
-		gtk_rc_parse (GdmGtkRC);
+	if ( ! inhibit_gtk_themes) {
+		const char *theme_name;
+
+		if ( ! ve_string_empty (GdmGtkRC) &&
+		     access (GdmGtkRC, R_OK) == 0)
+			gtk_rc_parse (GdmGtkRC);
+
+		theme_name = d->theme_name;
+		if (ve_string_empty (theme_name))
+			theme_name = GdmGtkTheme;
+		if ( ! ve_string_empty (theme_name)) {
+			gchar *theme_dir = gtk_rc_get_theme_dir ();
+			char *theme = g_strdup_printf ("%s/%s/gtk-2.0/gtkrc", theme_dir, theme_name);
+			g_free (theme_dir);
+
+			if ( ! ve_string_empty (theme) &&
+			     access (theme, R_OK) == 0)
+				gtk_rc_parse (theme);
+
+			g_free (theme);
+		}
+	}
 
 	get_screen_size (d);
 }
@@ -327,11 +348,11 @@ gdm_error_box_full (GdmDisplay *d, GtkMessageType type, const char *error,
 			GString *gs = g_string_new (NULL);
 
 			fp = NULL;
-			IGNORE_EINTR (r = lstat (details_file, &s));
+			VE_IGNORE_EINTR (r = lstat (details_file, &s));
 			if (r == 0) {
-				if (S_ISREG (s.st_mode))
-					fp = fopen (details_file, "r");
-				else {
+				if (S_ISREG (s.st_mode)) {
+					VE_IGNORE_EINTR (fp = fopen (details_file, "r"));
+				} else {
 					loc = gdm_locale_to_utf8 (_("%s not a regular file!\n"));
 					g_string_printf (gs, loc, details_file);
 					g_free (loc);
@@ -340,7 +361,9 @@ gdm_error_box_full (GdmDisplay *d, GtkMessageType type, const char *error,
 			if (fp != NULL) {
 				char buf[256];
 				int lines = 0;
-				while (fgets (buf, sizeof (buf), fp)) {
+				char *getsret;
+				VE_IGNORE_EINTR (getsret = fgets (buf, sizeof (buf), fp));
+				while (getsret != NULL) {
 					if ( ! g_utf8_validate (buf, -1, NULL))
 						valid_utf8 = FALSE;
 					g_string_append (gs, buf);
@@ -352,8 +375,9 @@ gdm_error_box_full (GdmDisplay *d, GtkMessageType type, const char *error,
 						g_free (loc);
 						break;
 					}
+					VE_IGNORE_EINTR (getsret = fgets (buf, sizeof (buf), fp));
 				}
-				fclose (fp);
+				VE_IGNORE_EINTR (fclose (fp));
 			} else {
 				loc = gdm_locale_to_utf8 (_("%s could not be opened"));
 				g_string_append_printf (gs, loc, details_file);
@@ -577,13 +601,13 @@ gdm_failsafe_question (GdmDisplay *d,
 		char buf[BUFSIZ];
 		int bytes;
 
-		IGNORE_EINTR (close (p[1]));
+		VE_IGNORE_EINTR (close (p[1]));
 
 		gdm_wait_for_extra (&status);
 
 		if (dialog_failed (status)) {
 			char *ret = NULL;
-			IGNORE_EINTR (close (p[0]));
+			VE_IGNORE_EINTR (close (p[0]));
 			if ( ! inhibit_gtk_themes) {
 				/* on failure try again, this time without any themes
 				   which may be causing a crash */
@@ -600,13 +624,13 @@ gdm_failsafe_question (GdmDisplay *d,
 			return ret;
 		}
 
-		IGNORE_EINTR (bytes = read (p[0], buf, BUFSIZ-1));
+		VE_IGNORE_EINTR (bytes = read (p[0], buf, BUFSIZ-1));
 		if (bytes > 0) {
-			IGNORE_EINTR (close (p[0]));
+			VE_IGNORE_EINTR (close (p[0]));
 			buf[bytes] = '\0';
 			return g_strdup (buf);
 		} 
-		IGNORE_EINTR (close (p[0]));
+		VE_IGNORE_EINTR (close (p[0]));
 	} else {
 		gdm_error (_("%s: Cannot fork to display error/info box"),
 			   "gdm_failsafe_question");
@@ -683,13 +707,13 @@ gdm_failsafe_yesno (GdmDisplay *d,
 		char buf[BUFSIZ];
 		int bytes;
 
-		IGNORE_EINTR (close (p[1]));
+		VE_IGNORE_EINTR (close (p[1]));
 
 		gdm_wait_for_extra (&status);
 
 		if (dialog_failed (status)) {
 			gboolean ret = FALSE;
-			IGNORE_EINTR (close (p[0]));
+			VE_IGNORE_EINTR (close (p[0]));
 			if ( ! inhibit_gtk_themes) {
 				/* on failure try again, this time without any themes
 				   which may be causing a crash */
@@ -706,15 +730,15 @@ gdm_failsafe_yesno (GdmDisplay *d,
 			return ret;
 		}
 
-		IGNORE_EINTR (bytes = read (p[0], buf, BUFSIZ-1));
+		VE_IGNORE_EINTR (bytes = read (p[0], buf, BUFSIZ-1));
 		if (bytes > 0) {
-			IGNORE_EINTR (close (p[0]));
+			VE_IGNORE_EINTR (close (p[0]));
 			if (buf[0] == 'y')
 				return TRUE;
 			else
 				return FALSE;
 		} 
-		IGNORE_EINTR (close (p[0]));
+		VE_IGNORE_EINTR (close (p[0]));
 	} else {
 		gdm_error (_("%s: Cannot fork to display error/info box"),
 			   "gdm_failsafe_yesno");
@@ -799,13 +823,13 @@ gdm_failsafe_ask_buttons (GdmDisplay *d,
 		char buf[BUFSIZ];
 		int bytes;
 
-		IGNORE_EINTR (close (p[1]));
+		VE_IGNORE_EINTR (close (p[1]));
 
 		gdm_wait_for_extra (&status);
 
 		if (dialog_failed (status)) {
 			int ret = -1;
-			IGNORE_EINTR (close (p[0]));
+			VE_IGNORE_EINTR (close (p[0]));
 			if ( ! inhibit_gtk_themes) {
 				/* on failure try again, this time without any themes
 				   which may be causing a crash */
@@ -822,17 +846,17 @@ gdm_failsafe_ask_buttons (GdmDisplay *d,
 			return ret;
 		}
 
-		IGNORE_EINTR (bytes = read (p[0], buf, BUFSIZ-1));
+		VE_IGNORE_EINTR (bytes = read (p[0], buf, BUFSIZ-1));
 		if (bytes > 0) {
 			int i;
-			IGNORE_EINTR (close (p[0]));
+			VE_IGNORE_EINTR (close (p[0]));
 			buf[bytes] = '\0';
 			if (sscanf (buf, "%d", &i) == 1)
 				return i;
 			else
 				return -1;
 		} 
-		IGNORE_EINTR (close (p[0]));
+		VE_IGNORE_EINTR (close (p[0]));
 	} else {
 		gdm_error (_("%s: Cannot fork to display error/info box"),
 			   "gdm_failsafe_ask_buttons");
